@@ -3,24 +3,24 @@
 use crate::{DhcpV4Client, DhcpV4Config, DhcpV4Lease};
 
 use super::env::{
-    DhcpServerEnv, TEST_NIC_CLI, TEST_PROXY_IP1, TEST_PROXY_MAC1,
+    with_dhcp_env, TEST_NIC_CLI, TEST_PROXY_IP1, TEST_PROXY_MAC1,
 };
 
 const POLL_WAIT_TIME: u32 = 5;
 
 #[test]
 fn test_dhcpv4_proxy() {
-    let _srv = DhcpServerEnv::start();
+    with_dhcp_env(|| {
+        let config = DhcpV4Config::new_proxy(TEST_NIC_CLI, TEST_PROXY_MAC1);
+        let mut cli = DhcpV4Client::init(config, None).unwrap();
 
-    let config = DhcpV4Config::new_proxy(TEST_NIC_CLI, TEST_PROXY_MAC1);
-    let mut cli = DhcpV4Client::init(config, None).unwrap();
-
-    let lease = get_lease(&mut cli);
-    assert!(lease.is_some());
-    if let Some(lease) = lease {
-        assert_eq!(lease.yiaddr, TEST_PROXY_IP1);
-        cli.release(&lease).unwrap();
-    }
+        let lease = get_lease(&mut cli);
+        assert!(lease.is_some());
+        if let Some(lease) = lease {
+            assert_eq!(lease.yiaddr, TEST_PROXY_IP1);
+            cli.release(&lease).unwrap();
+        }
+    })
 }
 
 fn get_lease(cli: &mut DhcpV4Client) -> Option<DhcpV4Lease> {
