@@ -13,8 +13,7 @@ pub(crate) fn mac_str_to_u8_array(mac: &str) -> Vec<u8> {
             Ok(i) => mac_bytes.push(i),
             Err(e) => {
                 error!(
-                    "Failed to convert to MAC address to bytes {:?}: {}",
-                    mac, e
+                    "Failed to convert to MAC address to bytes {mac:?}: {e}"
                 );
                 return Vec::new();
             }
